import	os
import	unittest
from	typing								import Callable
from	pygwarts.tests.magical				import MagicalTestCase
from	pygwarts.magical.philosophers_stone	import Transmutable
from	pygwarts.magical.chests				import KeyChest
from	pygwarts.irma.contrib				import LibraryContrib








class KeyChestCase(MagicalTestCase):

	"""
		Test cases for magical.chests.KeyChest
	"""

	@classmethod
	def tearDownClass(cls):

		if	cls.clean_up:
			if	os.path.isfile(cls.KEY_SET_HANDLER): os.remove(cls.KEY_SET_HANDLER)

	@classmethod
	def setUpClass(cls):
		class Treasure(Transmutable):

			weight	:int
			__call__:Callable[[], str]

		class CurrentChest(KeyChest):
			class loggy(LibraryContrib):

				init_name	= "key_chest"
				init_level	= 10
				handler		= cls.KEY_SET_HANDLER

			class Diamond(Treasure):

				weight	= 150
				def __call__(self): return "is forever?"

			class Ruby(Treasure):

				weight	= 150
				def __call__(self): return "shining red"

			class Jade(Treasure):

				weight	= 200
				def __call__(self): return "ZYO"

			class Sapphire(Treasure):

				weight	= 174
				def __call__(self): return "expense"

			class Gold(Treasure):

				weight	= 995
				def __call__(self): return "The Best"

			class Silver(Treasure):

				weight	= 100500
				def __call__(self): return "never too much"

			class Boot(Transmutable):

				weight	= 500
				def __call__(self): return "was caught in a river, so might be dear to heart"


		cls.make_loggy_file(cls, cls.KEY_SET_HANDLER)
		cls.test_case = CurrentChest()
		cls.test_case(cls.test_case.Diamond,	cls.test_case.Diamond.weight)
		cls.test_case(cls.test_case.Ruby,		cls.test_case.Ruby.weight)
		cls.test_case(cls.test_case.Jade,		cls.test_case.Jade.weight)
		cls.test_case(cls.test_case.Sapphire,	cls.test_case.Sapphire.weight)
		cls.test_case(cls.test_case.Gold,		cls.test_case.Gold.weight)
		cls.test_case(cls.test_case.Silver,		cls.test_case.Silver.weight)

		class SameChest(KeyChest):	pass
		class NotSameChest(KeyChest):
			class Diamond(Treasure):

				weight	= 1002
				def __call__(self): return "same is forever?"

			class Ruby(Treasure):

				weight	= 1502
				def __call__(self): return "same shining red"

			class Jade(Treasure):

				weight	= 2002
				def __call__(self): return "same ZYO"

			class Sapphire(Treasure):

				weight	= 1742
				def __call__(self): return "same expense"

			class Gold(Treasure):

				weight	= 9952
				def __call__(self): return "same The Best"

			class Silver(Treasure):

				weight	= 1005002
				def __call__(self): return "same never too much"

			class Boot(Transmutable):

				weight	= 5002
				def __call__(self): return "same was caught in a river, so might be dear to heart"

		cls.not_same_case = NotSameChest()
		cls.not_same_case(cls.not_same_case.Diamond,	cls.not_same_case.Diamond.weight)
		cls.not_same_case(cls.not_same_case.Ruby,		cls.not_same_case.Ruby.weight)
		cls.not_same_case(cls.not_same_case.Jade,		cls.not_same_case.Jade.weight)
		cls.not_same_case(cls.not_same_case.Sapphire,	cls.not_same_case.Sapphire.weight)
		cls.not_same_case(cls.not_same_case.Gold,		cls.not_same_case.Gold.weight)
		cls.not_same_case(cls.not_same_case.Silver,		cls.not_same_case.Silver.weight)

		cls.same_case = SameChest()
		cls.same_case(cls.test_case.Diamond,	cls.test_case.Diamond.weight)
		cls.same_case(cls.test_case.Ruby,		cls.test_case.Ruby.weight)
		cls.same_case(cls.test_case.Jade,		cls.test_case.Jade.weight)
		cls.same_case(cls.test_case.Sapphire,	cls.test_case.Sapphire.weight)
		cls.same_case(cls.test_case.Gold,		cls.test_case.Gold.weight)
		cls.same_case(cls.test_case.Silver,		cls.test_case.Silver.weight)

		cls.same_asdict = {

			cls.test_case.Diamond:	cls.test_case.Diamond.weight,
			cls.test_case.Ruby:		cls.test_case.Ruby.weight,
			cls.test_case.Jade:		cls.test_case.Jade.weight,
			cls.test_case.Sapphire:	cls.test_case.Sapphire.weight,
			cls.test_case.Gold:		cls.test_case.Gold.weight,
			cls.test_case.Silver:	cls.test_case.Silver.weight
		}
		cls.not_same_asdict = {

			cls.not_same_case.Diamond:	cls.not_same_case.Diamond.weight,
			cls.not_same_case.Ruby:		cls.not_same_case.Ruby.weight,
			cls.not_same_case.Jade:		cls.not_same_case.Jade.weight,
			cls.not_same_case.Sapphire:	cls.not_same_case.Sapphire.weight,
			cls.not_same_case.Gold:		cls.not_same_case.Gold.weight,
			cls.not_same_case.Silver:	cls.not_same_case.Silver.weight
		}




	def test_length(self): self.assertEqual(len(self.test_case), 6)
	def test_contains(self):

		self.assertIn(self.test_case.Diamond,	self.test_case)
		self.assertIn(self.test_case.Ruby,		self.test_case)
		self.assertIn(self.test_case.Jade,		self.test_case)
		self.assertIn(self.test_case.Sapphire,	self.test_case)
		self.assertIn(self.test_case.Gold,		self.test_case)
		self.assertIn(self.test_case.Silver,	self.test_case)
		self.assertNotIn(self.test_case.Boot,	self.test_case)




	def test_getitem(self):

		self.assertEqual(self.test_case[self.test_case.Diamond],	self.test_case.Diamond.weight)
		self.assertEqual(self.test_case[self.test_case.Ruby],		self.test_case.Ruby.weight)
		self.assertEqual(self.test_case[self.test_case.Jade],		self.test_case.Jade.weight)
		self.assertEqual(self.test_case[self.test_case.Sapphire],	self.test_case.Sapphire.weight)
		self.assertEqual(self.test_case[self.test_case.Gold],		self.test_case.Gold.weight)
		self.assertEqual(self.test_case[self.test_case.Silver],		self.test_case.Silver.weight)
		self.assertEqual(self.test_case[self.test_case.Boot],		None)




	def test_setitem_and_delitem(self):

		self.assertNotIn(self.test_case.Boot, self.test_case)
		self.assertIn(self.test_case.Sapphire, self.test_case)
		self.assertEqual(self.test_case[self.test_case.Sapphire], self.test_case.Sapphire.weight)

		self.test_case[self.test_case.Sapphire] = 69
		self.test_case[self.test_case.Boot] = self.test_case.Boot.weight

		self.assertIn(self.test_case.Boot, self.test_case)
		self.assertEqual(self.test_case[self.test_case.Boot], self.test_case.Boot.weight)
		self.assertNotEqual(self.test_case[self.test_case.Sapphire], self.test_case.Sapphire.weight)

		with self.assertLogs("key_chest", 30) as case_loggy : self.test_case[[self.test_case.Sapphire]] = 42

		self.assertIn(

			f"WARNING:key_chest:Can't put \"{[self.test_case.Sapphire]}\", "
			f"\"{42}\" pair in {self.test_case} KeyChest",
			case_loggy.output
		)

		self.test_case[self.test_case.Sapphire] = self.test_case.Sapphire.weight
		del self.test_case[self.test_case.Boot]

		self.assertNotIn(self.test_case.Boot, self.test_case)
		self.assertIn(self.test_case.Sapphire, self.test_case)
		self.assertEqual(self.test_case[self.test_case.Sapphire], self.test_case.Sapphire.weight)


		# Broken "_locker_" must make some raise resulting warning log message
		locker = self.test_case._locker_[:]
		self.test_case._locker_ = []
		with self.assertLogs("key_chest", 30) as case_loggy: del self.test_case[self.test_case.Gold]
		self.assertIn(
			f"WARNING:key_chest:Key \"{self.test_case.Gold}\" not found in locker", case_loggy.output
		)
		self.test_case._locker_ = locker[:]




	def test_loading_and_deleting(self):

		self.test_case(self.test_case.Boot, self.test_case.Boot.weight)
		self.assertEqual(len(self.test_case), 7)
		self.assertEqual(self.test_case[self.test_case.Diamond], 	self.test_case.Diamond.weight)
		self.assertEqual(self.test_case[self.test_case.Ruby], 		self.test_case.Ruby.weight)
		self.assertEqual(self.test_case[self.test_case.Jade], 		self.test_case.Jade.weight)
		self.assertEqual(self.test_case[self.test_case.Sapphire], 	self.test_case.Sapphire.weight)
		self.assertEqual(self.test_case[self.test_case.Gold], 		self.test_case.Gold.weight)
		self.assertEqual(self.test_case[self.test_case.Silver], 	self.test_case.Silver.weight)
		self.assertEqual(self.test_case[self.test_case.Boot], 		self.test_case.Boot.weight)

		del self.test_case[self.test_case.Boot]
		self.assertNotIn(self.test_case.Boot, self.test_case)
		self.assertEqual(len(self.test_case), 6)




	def test_nested_loading_and_deleting(self):

		self.test_case("deep", {})
		self.test_case("deeper", {}, "deep")
		self.test_case("deepest", {}, "deep", "deeper")
		self.assertIn("deep", self.test_case)
		self.assertIn("deeper", self.test_case["deep"])
		self.assertIn("deepest", self.test_case["deep"]["deeper"])


		self.test_case(self.test_case.Boot, self.test_case.Boot.weight, "deep")
		self.test_case(self.test_case.Boot, self.test_case.Boot.weight, "deep", "deeper")
		self.test_case(self.test_case.Boot, self.test_case.Boot.weight, "deep", "deeper", "deepest")
		self.assertEqual(len(self.test_case), 7)
		self.assertIn(self.test_case.Boot, self.test_case["deep"])
		self.assertIn(self.test_case.Boot, self.test_case["deep"]["deeper"])
		self.assertIn(self.test_case.Boot, self.test_case["deep"]["deeper"]["deepest"])
		self.assertEqual(
			self.test_case["deep"][self.test_case.Boot], self.test_case.Boot.weight
		)
		self.assertEqual(
			self.test_case["deep"]["deeper"][self.test_case.Boot], self.test_case.Boot.weight
		)
		self.assertEqual(
			self.test_case["deep"]["deeper"]["deepest"][self.test_case.Boot], self.test_case.Boot.weight
		)


		del self.test_case["deep"]
		self.assertEqual(len(self.test_case), 6)


		self.test_case(
			self.test_case.Boot, self.test_case.Boot.weight, "deep", mapped=False
		)
		self.test_case(
			self.test_case.Boot, self.test_case.Boot.weight, "deep", "deeper", mapped=False
		)
		self.test_case(
			self.test_case.Boot, self.test_case.Boot.weight, "deep", "deeper", "deepest", mapped=False
		)
		self.assertEqual(len(self.test_case), 7)
		self.assertIn(self.test_case.Boot, self.test_case["deep"])
		self.assertIn(self.test_case.Boot, self.test_case["deep"]["deeper"])
		self.assertIn(self.test_case.Boot, self.test_case["deep"]["deeper"]["deepest"])
		self.assertEqual(
			self.test_case["deep"][self.test_case.Boot], self.test_case.Boot.weight
		)
		self.assertEqual(
			self.test_case["deep"]["deeper"][self.test_case.Boot], self.test_case.Boot.weight
		)
		self.assertEqual(
			self.test_case["deep"]["deeper"]["deepest"][self.test_case.Boot], self.test_case.Boot.weight
		)


		del self.test_case["deep"]
		self.assertEqual(len(self.test_case), 6)




	def test_unload(self):

		self.assertEqual(len(self.test_case), 6)
		self.assertEqual(self.test_case[self.test_case.Diamond],	self.test_case.Diamond.weight)
		self.assertEqual(self.test_case[self.test_case.Ruby],		self.test_case.Ruby.weight)
		self.assertEqual(self.test_case[self.test_case.Jade],		self.test_case.Jade.weight)
		self.assertEqual(self.test_case[self.test_case.Sapphire],	self.test_case.Sapphire.weight)
		self.assertEqual(self.test_case[self.test_case.Gold],		self.test_case.Gold.weight)
		self.assertEqual(self.test_case[self.test_case.Silver],		self.test_case.Silver.weight)

		self.test_case.unload()
		self.assertEqual(len(self.test_case), 0)
		self.assertNotIn(self.test_case.Diamond,	self.test_case)
		self.assertNotIn(self.test_case.Ruby,		self.test_case)
		self.assertNotIn(self.test_case.Jade,		self.test_case)
		self.assertNotIn(self.test_case.Sapphire,	self.test_case)
		self.assertNotIn(self.test_case.Gold,		self.test_case)
		self.assertNotIn(self.test_case.Silver,		self.test_case)
		self.assertNotIn(self.test_case.Boot,		self.test_case)
		self.assertEqual(self.test_case[self.test_case.Diamond],	None)
		self.assertEqual(self.test_case[self.test_case.Ruby],		None)
		self.assertEqual(self.test_case[self.test_case.Jade],		None)
		self.assertEqual(self.test_case[self.test_case.Sapphire],	None)
		self.assertEqual(self.test_case[self.test_case.Gold],		None)
		self.assertEqual(self.test_case[self.test_case.Silver],		None)

		self.test_case(self.test_case.Diamond,	self.test_case.Diamond.weight)
		self.test_case(self.test_case.Ruby,		self.test_case.Ruby.weight)
		self.test_case(self.test_case.Jade,		self.test_case.Jade.weight)
		self.test_case(self.test_case.Sapphire,	self.test_case.Sapphire.weight)
		self.test_case(self.test_case.Gold,		self.test_case.Gold.weight)
		self.test_case(self.test_case.Silver,	self.test_case.Silver.weight)
		self.assertEqual(len(self.test_case), 6)
		self.assertEqual(self.test_case[self.test_case.Diamond],	self.test_case.Diamond.weight)
		self.assertEqual(self.test_case[self.test_case.Ruby],		self.test_case.Ruby.weight)
		self.assertEqual(self.test_case[self.test_case.Jade],		self.test_case.Jade.weight)
		self.assertEqual(self.test_case[self.test_case.Sapphire],	self.test_case.Sapphire.weight)
		self.assertEqual(self.test_case[self.test_case.Gold],		self.test_case.Gold.weight)
		self.assertEqual(self.test_case[self.test_case.Silver],		self.test_case.Silver.weight)




	def test_keysof(self):

		self.assertEqual(

			self.test_case.keysof(self.test_case.Diamond.weight),
			[ self.test_case.Diamond, self.test_case.Ruby ]
		)
		self.assertEqual(

			self.test_case.keysof(self.test_case.Ruby.weight),
			[ self.test_case.Diamond, self.test_case.Ruby ]
		)
		self.assertEqual(self.test_case.keysof(self.test_case.Jade.weight),		[ self.test_case.Jade ])
		self.assertEqual(self.test_case.keysof(self.test_case.Sapphire.weight),	[ self.test_case.Sapphire ])
		self.assertEqual(self.test_case.keysof(self.test_case.Gold.weight),		[ self.test_case.Gold ])
		self.assertEqual(self.test_case.keysof(self.test_case.Silver.weight),	[ self.test_case.Silver ])
		self.assertEqual(self.test_case.keysof(self.test_case.Boot.weight),		[])


		# Broken "_locker_" case
		locker = self.test_case._locker_[:]
		self.test_case._locker_ = []
		self.assertEqual(self.test_case.keysof(self.test_case.Diamond.weight),	[])
		self.assertEqual(self.test_case.keysof(self.test_case.Ruby.weight),		[])
		self.assertEqual(self.test_case.keysof(self.test_case.Jade.weight),		[])
		self.assertEqual(self.test_case.keysof(self.test_case.Sapphire.weight),	[])
		self.assertEqual(self.test_case.keysof(self.test_case.Gold.weight),		[])
		self.assertEqual(self.test_case.keysof(self.test_case.Silver.weight),	[])
		self.assertEqual(self.test_case.keysof(self.test_case.Boot.weight),		[])
		self.test_case._locker_ = locker[:]




	def test_eq_Chest(self):

		self.assertNotEqual(self.test_case, self.not_same_case)
		self.assertEqual(len(self.test_case), len(self.not_same_case))

		self.assertNotEqual(
			self.test_case[self.test_case.Diamond],		self.not_same_case[self.not_same_case.Diamond]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Ruby],		self.not_same_case[self.not_same_case.Ruby]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Jade],		self.not_same_case[self.not_same_case.Jade]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Sapphire],	self.not_same_case[self.not_same_case.Sapphire]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Gold],		self.not_same_case[self.not_same_case.Gold]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Silver],		self.not_same_case[self.not_same_case.Silver]
		)

		self.assertEqual(self.test_case, self.same_case)
		self.assertEqual(len(self.test_case), len(self.same_case))

		self.assertEqual(
			self.test_case[self.test_case.Diamond],		self.same_case[self.test_case.Diamond]
		)
		self.assertEqual(
			self.test_case[self.test_case.Ruby],		self.same_case[self.test_case.Ruby]
		)
		self.assertEqual(
			self.test_case[self.test_case.Jade],		self.same_case[self.test_case.Jade]
		)
		self.assertEqual(
			self.test_case[self.test_case.Sapphire],	self.same_case[self.test_case.Sapphire]
		)
		self.assertEqual(
			self.test_case[self.test_case.Gold],		self.same_case[self.test_case.Gold]
		)
		self.assertEqual(
			self.test_case[self.test_case.Silver],		self.same_case[self.test_case.Silver]
		)




	def test_gt_ge_lt_le_Chest(self):

		self.assertEqual(self.test_case[self.test_case.Diamond],	self.test_case.Diamond.weight)
		self.assertEqual(self.test_case[self.test_case.Ruby],		self.test_case.Ruby.weight)
		self.assertEqual(self.test_case[self.test_case.Jade],		self.test_case.Jade.weight)
		self.assertEqual(self.test_case[self.test_case.Sapphire],	self.test_case.Sapphire.weight)
		self.assertEqual(self.test_case[self.test_case.Gold],		self.test_case.Gold.weight)
		self.assertEqual(self.test_case[self.test_case.Silver],		self.test_case.Silver.weight)
		del self.not_same_case[self.not_same_case.Diamond]

		self.assertNotEqual(self.test_case, self.not_same_case)
		self.assertNotEqual(len(self.test_case), len(self.not_same_case))
		self.assertGreater(self.test_case, self.not_same_case)
		self.assertGreaterEqual(self.test_case, self.not_same_case)
		self.assertLess(self.not_same_case, self.test_case)
		self.assertLessEqual(self.not_same_case, self.test_case)

		self.not_same_case.unload()
		self.not_same_case(self.not_same_case.Diamond,	self.not_same_case.Diamond.weight)
		self.not_same_case(self.not_same_case.Ruby,		self.not_same_case.Ruby.weight)
		self.not_same_case(self.not_same_case.Jade,		self.not_same_case.Jade.weight)
		self.not_same_case(self.not_same_case.Sapphire,	self.not_same_case.Sapphire.weight)
		self.not_same_case(self.not_same_case.Gold,		self.not_same_case.Gold.weight)
		self.not_same_case(self.not_same_case.Silver,	self.not_same_case.Silver.weight)

		self.assertEqual(self.not_same_case[self.not_same_case.Diamond],	self.not_same_case.Diamond.weight)
		self.assertEqual(self.not_same_case[self.not_same_case.Ruby],		self.not_same_case.Ruby.weight)
		self.assertEqual(self.not_same_case[self.not_same_case.Jade],		self.not_same_case.Jade.weight)
		self.assertEqual(self.not_same_case[self.not_same_case.Sapphire],	self.not_same_case.Sapphire.weight)
		self.assertEqual(self.not_same_case[self.not_same_case.Gold],		self.not_same_case.Gold.weight)
		self.assertEqual(self.not_same_case[self.not_same_case.Silver],		self.not_same_case.Silver.weight)




	def test_eq_dict(self):

		self.assertNotEqual(self.test_case, self.not_same_asdict)
		self.assertEqual(len(self.test_case), len(self.not_same_asdict))

		self.assertNotEqual(
			self.test_case[self.test_case.Diamond],		self.not_same_asdict[self.not_same_case.Diamond]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Ruby],		self.not_same_asdict[self.not_same_case.Ruby]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Jade],		self.not_same_asdict[self.not_same_case.Jade]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Sapphire],	self.not_same_asdict[self.not_same_case.Sapphire]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Gold],		self.not_same_asdict[self.not_same_case.Gold]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Silver],		self.not_same_asdict[self.not_same_case.Silver]
		)


		self.assertEqual(self.test_case, self.same_asdict)
		self.assertEqual(len(self.test_case), len(self.same_asdict))

		self.assertEqual(
			self.test_case[self.test_case.Diamond],		self.same_asdict[self.test_case.Diamond]
		)
		self.assertEqual(
			self.test_case[self.test_case.Ruby],		self.same_asdict[self.test_case.Ruby]
		)
		self.assertEqual(
			self.test_case[self.test_case.Jade],		self.same_asdict[self.test_case.Jade]
		)
		self.assertEqual(
			self.test_case[self.test_case.Sapphire],	self.same_asdict[self.test_case.Sapphire]
		)
		self.assertEqual(
			self.test_case[self.test_case.Gold],		self.same_asdict[self.test_case.Gold]
		)
		self.assertEqual(
			self.test_case[self.test_case.Silver],		self.same_asdict[self.test_case.Silver]
		)




	def test_gt_ge_lt_le_list(self):

		self.assertEqual(self.not_same_asdict[self.not_same_case.Diamond],	self.not_same_case.Diamond.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Ruby],		self.not_same_case.Ruby.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Jade],		self.not_same_case.Jade.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Sapphire],	self.not_same_case.Sapphire.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Gold],		self.not_same_case.Gold.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Silver],	self.not_same_case.Silver.weight)
		del self.not_same_asdict[self.not_same_case.Diamond]

		self.assertNotEqual(self.test_case, self.not_same_asdict)
		self.assertNotEqual(len(self.test_case), len(self.not_same_asdict))
		self.assertGreater(self.test_case, self.not_same_asdict)
		self.assertGreaterEqual(self.test_case, self.not_same_asdict)
		self.assertLess(self.not_same_asdict, self.test_case)
		self.assertLessEqual(self.not_same_asdict, self.test_case)

		del self.not_same_asdict[self.not_same_case.Ruby]
		del self.not_same_asdict[self.not_same_case.Jade]
		del self.not_same_asdict[self.not_same_case.Sapphire]
		del self.not_same_asdict[self.not_same_case.Gold]
		del self.not_same_asdict[self.not_same_case.Silver]
		self.assertFalse(self.not_same_asdict)
		self.not_same_asdict[self.not_same_case.Diamond]	=	self.not_same_case.Diamond.weight
		self.not_same_asdict[self.not_same_case.Ruby]		=	self.not_same_case.Ruby.weight
		self.not_same_asdict[self.not_same_case.Jade]		=	self.not_same_case.Jade.weight
		self.not_same_asdict[self.not_same_case.Sapphire]	=	self.not_same_case.Sapphire.weight
		self.not_same_asdict[self.not_same_case.Gold]		=	self.not_same_case.Gold.weight
		self.not_same_asdict[self.not_same_case.Silver]		=	self.not_same_case.Silver.weight

		self.assertEqual(self.not_same_asdict[self.not_same_case.Diamond],	self.not_same_case.Diamond.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Ruby],		self.not_same_case.Ruby.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Jade],		self.not_same_case.Jade.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Sapphire],	self.not_same_case.Sapphire.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Gold],		self.not_same_case.Gold.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Silver],	self.not_same_case.Silver.weight)




	def test_iter(self):

		count = 0
		for K,V in self.test_case:

			self.assertEqual(K.weight, V)
			self.assertIsInstance(K(), str)
			count += 1

		self.assertEqual(count, len(self.test_case))
		count = 0
		_case = iter(self.test_case)
		for one,two in zip(_case, _case):

			Kone,Vone = one
			self.assertEqual(Kone.weight, Vone)
			self.assertIsInstance(Kone(), str)
			count += 1
			Ktwo,Vtwo = two
			self.assertEqual(Ktwo.weight, Vtwo)
			self.assertIsInstance(Ktwo(), str)
			count += 1

		self.assertEqual(count, len(self.test_case))


		# Breaking iteration by messing with "_locker_":
		locker = self.test_case._locker_[:]
		self.test_case._locker_ = []
		count = 0
		for K,V in self.test_case:

			self.assertEqual(K.weight, V)
			self.assertIsInstance(K(), str)
			count += 1

		self.assertEqual(count, 0)
		self.assertEqual(len(self.test_case), 6)
		self.assertNotEqual(count, len(self.test_case))
		self.test_case._locker_ = locker[:]




	def test_next(self):

		count = 0
		for _ in range(3 *len(self.test_case)):

			try:

				K,V = next(self.test_case)
				self.assertEqual(K.weight, V)
				self.assertIsInstance(K(), str)
				count += 1

			except	StopIteration:

				self.assertEqual(count, len(self.test_case))
				count = 0


		# Breaking "_locker_" and __next__ always raises StopIteration
		locker = self.test_case._locker_[:]
		self.test_case._locker_ = []
		count = 0
		for _ in range(3):

			try:

				next(self.test_case)
				count += 1

			except	StopIteration : self.assertEqual(count,0)

		self.assertEqual(count,0)
		self.test_case._locker_ = locker[:]




	def test_reversed(self):

		# Direct
		self.assertEqual(
			[ V for K,V in self.test_case ],
			[
				self.test_case.Diamond.weight,
				self.test_case.Ruby.weight,
				self.test_case.Jade.weight,
				self.test_case.Sapphire.weight,
				self.test_case.Gold.weight,
				self.test_case.Silver.weight
			]
		)

		# Reversed
		self.assertEqual(
			[ V for K,V in reversed(self.test_case) ],
			[
				self.test_case.Silver.weight,
				self.test_case.Gold.weight,
				self.test_case.Sapphire.weight,
				self.test_case.Jade.weight,
				self.test_case.Ruby.weight,
				self.test_case.Diamond.weight
			]
		)

		# Broken "_locker_" that already reversed
		locker = self.test_case._locker_[:]
		self.test_case._locker_ = list(reversed(self.test_case._locker_))
		self.assertEqual(
			[ V for K,V in reversed(self.test_case) ],
			[
				self.test_case.Diamond.weight,
				self.test_case.Ruby.weight,
				self.test_case.Jade.weight,
				self.test_case.Sapphire.weight,
				self.test_case.Gold.weight,
				self.test_case.Silver.weight
			]
		)

		# Broken "_locker_" that is empty
		self.test_case._locker_ = []
		self.assertEqual([ V for K,V in self.test_case ], [])
		self.test_case._locker_ = locker[:]




	def test_noload_call(self):

		test_case = self.test_case()
		self.assertIsInstance(test_case, dict)
		self.assertEqual(test_case, self.test_case())
		self.assertNotEqual(id(test_case), id(self.test_case))

		self.assertEqual(self.test_case[self.test_case.Diamond],	test_case[str(self.test_case.Diamond)])
		self.assertEqual(self.test_case[self.test_case.Ruby],		test_case[str(self.test_case.Ruby)])
		self.assertEqual(self.test_case[self.test_case.Jade],		test_case[str(self.test_case.Jade)])
		self.assertEqual(self.test_case[self.test_case.Sapphire],	test_case[str(self.test_case.Sapphire)])
		self.assertEqual(self.test_case[self.test_case.Gold],		test_case[str(self.test_case.Gold)])
		self.assertEqual(self.test_case[self.test_case.Silver],		test_case[str(self.test_case.Silver)])
















class KeySelfPackedCase(MagicalTestCase):

	"""
		Test cases for magical.chests.Chest
	"""

	@classmethod
	def tearDownClass(cls):

		if	cls.clean_up:
			if	os.path.isfile(cls.PACKED_KEY_SET_HANDLER): os.remove(cls.PACKED_KEY_SET_HANDLER)

	@classmethod
	def setUpClass(cls):
		class Treasure(Transmutable):

			weight	:int
			__call__:Callable[[], str]

			def __init__(self, *args, **kwargs):

				super().__init__(*args, **kwargs)
				if	isinstance(self._UPPER_LAYER, KeyChest): self._UPPER_LAYER(self, self.weight)

		class CurrentChest(KeyChest):
			class loggy(LibraryContrib):

				init_name	= "pkey_chest"
				init_level	= 10
				handler		= cls.PACKED_KEY_SET_HANDLER

			class Diamond(Treasure):

				weight	= 150
				def __call__(self): return "is forever?"

			class Ruby(Treasure):

				weight	= 150
				def __call__(self): return "shining red"

			class Jade(Treasure):

				weight	= 200
				def __call__(self): return "ZYO"

			class Sapphire(Treasure):

				weight	= 174
				def __call__(self): return "expense"

			class Gold(Treasure):

				weight	= 995
				def __call__(self): return "The Best"

			class Silver(Treasure):

				weight	= 100500
				def __call__(self): return "never too much"

			class Boot(Transmutable):

				weight	= 500
				def __call__(self): return "was caught in a river, so might be dear to heart"


		cls.make_loggy_file(cls, cls.PACKED_KEY_SET_HANDLER)
		cls.test_case = CurrentChest()

		class SameChest(KeyChest):	pass
		class NotSameChest(KeyChest):
			class Diamond(Treasure):

				weight	= 1002
				def __call__(self): return "same is forever?"

			class Ruby(Treasure):

				weight	= 1502
				def __call__(self): return "same shining red"

			class Jade(Treasure):

				weight	= 2002
				def __call__(self): return "same ZYO"

			class Sapphire(Treasure):

				weight	= 1742
				def __call__(self): return "same expense"

			class Gold(Treasure):

				weight	= 9952
				def __call__(self): return "same The Best"

			class Silver(Treasure):

				weight	= 1005002
				def __call__(self): return "same never too much"

			class Boot(Transmutable):

				weight	= 5002
				def __call__(self): return "same was caught in a river, so might be dear to heart"

		cls.not_same_case = NotSameChest()
		cls.same_case = SameChest()
		cls.same_case(cls.test_case.Diamond,	cls.test_case.Diamond.weight)
		cls.same_case(cls.test_case.Ruby,		cls.test_case.Ruby.weight)
		cls.same_case(cls.test_case.Jade,		cls.test_case.Jade.weight)
		cls.same_case(cls.test_case.Sapphire,	cls.test_case.Sapphire.weight)
		cls.same_case(cls.test_case.Gold,		cls.test_case.Gold.weight)
		cls.same_case(cls.test_case.Silver,		cls.test_case.Silver.weight)

		cls.same_asdict = {

			cls.test_case.Diamond:	cls.test_case.Diamond.weight,
			cls.test_case.Ruby:		cls.test_case.Ruby.weight,
			cls.test_case.Jade:		cls.test_case.Jade.weight,
			cls.test_case.Sapphire:	cls.test_case.Sapphire.weight,
			cls.test_case.Gold:		cls.test_case.Gold.weight,
			cls.test_case.Silver:	cls.test_case.Silver.weight
		}
		cls.not_same_asdict = {

			cls.not_same_case.Diamond:	cls.not_same_case.Diamond.weight,
			cls.not_same_case.Ruby:		cls.not_same_case.Ruby.weight,
			cls.not_same_case.Jade:		cls.not_same_case.Jade.weight,
			cls.not_same_case.Sapphire:	cls.not_same_case.Sapphire.weight,
			cls.not_same_case.Gold:		cls.not_same_case.Gold.weight,
			cls.not_same_case.Silver:	cls.not_same_case.Silver.weight
		}




	def test_packed_length(self):	self.assertEqual(len(self.test_case), 6)
	def test_packed_keys(self):		self.assertEqual(self.test_case.keys(), self.test_case._locker_)
	def test_packed_contains(self):

		self.assertIn(self.test_case.Diamond,	self.test_case)
		self.assertIn(self.test_case.Ruby,		self.test_case)
		self.assertIn(self.test_case.Jade,		self.test_case)
		self.assertIn(self.test_case.Sapphire,	self.test_case)
		self.assertIn(self.test_case.Gold,		self.test_case)
		self.assertIn(self.test_case.Silver,	self.test_case)
		self.assertNotIn(self.test_case.Boot,	self.test_case)




	def test_packed_getitem(self):

		self.assertEqual(self.test_case[self.test_case.Diamond],	self.test_case.Diamond.weight)
		self.assertEqual(self.test_case[self.test_case.Ruby],		self.test_case.Ruby.weight)
		self.assertEqual(self.test_case[self.test_case.Jade],		self.test_case.Jade.weight)
		self.assertEqual(self.test_case[self.test_case.Sapphire],	self.test_case.Sapphire.weight)
		self.assertEqual(self.test_case[self.test_case.Gold],		self.test_case.Gold.weight)
		self.assertEqual(self.test_case[self.test_case.Silver],		self.test_case.Silver.weight)
		self.assertEqual(self.test_case[self.test_case.Boot],		None)




	def test_packed_setitem_and_delitem(self):

		self.assertNotIn(self.test_case.Boot, self.test_case)
		self.assertIn(self.test_case.Sapphire, self.test_case)
		self.assertEqual(self.test_case[self.test_case.Sapphire], self.test_case.Sapphire.weight)

		self.test_case[self.test_case.Sapphire] = 69
		self.test_case[self.test_case.Boot] = self.test_case.Boot.weight

		self.assertIn(self.test_case.Boot, self.test_case)
		self.assertEqual(self.test_case[self.test_case.Boot], self.test_case.Boot.weight)
		self.assertNotEqual(self.test_case[self.test_case.Sapphire], self.test_case.Sapphire.weight)

		with self.assertLogs("pkey_chest", 30) as case_loggy : self.test_case[[self.test_case.Sapphire]] = 42

		self.assertIn(

			f"WARNING:pkey_chest:Can't put \"{[self.test_case.Sapphire]}\", "
			f"\"{42}\" pair in {self.test_case} KeyChest",
			case_loggy.output
		)

		self.test_case[self.test_case.Sapphire] = self.test_case.Sapphire.weight
		del self.test_case[self.test_case.Boot]

		self.assertNotIn(self.test_case.Boot, self.test_case)
		self.assertIn(self.test_case.Sapphire, self.test_case)
		self.assertEqual(self.test_case[self.test_case.Sapphire], self.test_case.Sapphire.weight)


		# Broken "_locker_" must make some raise resulting warning log message
		locker = self.test_case._locker_[:]
		self.test_case._locker_ = []
		with self.assertLogs("pkey_chest", 30) as case_loggy: del self.test_case[self.test_case.Gold]
		self.assertIn(
			f"WARNING:pkey_chest:Key \"{self.test_case.Gold}\" not found in locker", case_loggy.output
		)
		self.test_case._locker_ = locker[:]




	def test_packed_loading_and_deleting(self):

		self.test_case(self.test_case.Boot, self.test_case.Boot.weight)
		self.assertEqual(len(self.test_case), 7)
		self.assertEqual(self.test_case[self.test_case.Diamond], 	self.test_case.Diamond.weight)
		self.assertEqual(self.test_case[self.test_case.Ruby], 		self.test_case.Ruby.weight)
		self.assertEqual(self.test_case[self.test_case.Jade], 		self.test_case.Jade.weight)
		self.assertEqual(self.test_case[self.test_case.Sapphire], 	self.test_case.Sapphire.weight)
		self.assertEqual(self.test_case[self.test_case.Gold], 		self.test_case.Gold.weight)
		self.assertEqual(self.test_case[self.test_case.Silver], 	self.test_case.Silver.weight)
		self.assertEqual(self.test_case[self.test_case.Boot], 		self.test_case.Boot.weight)

		del self.test_case[self.test_case.Boot]
		self.assertNotIn(self.test_case.Boot, self.test_case)
		self.assertEqual(len(self.test_case), 6)




	def test_packed_nested_loading_and_deleting(self):

		self.test_case("deep", {})
		self.test_case("deeper", {}, "deep")
		self.test_case("deepest", {}, "deep", "deeper")
		self.assertIn("deep", self.test_case)
		self.assertIn("deeper", self.test_case["deep"])
		self.assertIn("deepest", self.test_case["deep"]["deeper"])


		self.test_case(self.test_case.Boot, self.test_case.Boot.weight, "deep")
		self.test_case(self.test_case.Boot, self.test_case.Boot.weight, "deep", "deeper")
		self.test_case(self.test_case.Boot, self.test_case.Boot.weight, "deep", "deeper", "deepest")
		self.assertEqual(len(self.test_case), 7)
		self.assertIn(self.test_case.Boot, self.test_case["deep"])
		self.assertIn(self.test_case.Boot, self.test_case["deep"]["deeper"])
		self.assertIn(self.test_case.Boot, self.test_case["deep"]["deeper"]["deepest"])
		self.assertEqual(
			self.test_case["deep"][self.test_case.Boot], self.test_case.Boot.weight
		)
		self.assertEqual(
			self.test_case["deep"]["deeper"][self.test_case.Boot], self.test_case.Boot.weight
		)
		self.assertEqual(
			self.test_case["deep"]["deeper"]["deepest"][self.test_case.Boot], self.test_case.Boot.weight
		)


		del self.test_case["deep"]
		self.assertEqual(len(self.test_case), 6)


		self.test_case(
			self.test_case.Boot, self.test_case.Boot.weight, "deep", mapped=False
		)
		self.test_case(
			self.test_case.Boot, self.test_case.Boot.weight, "deep", "deeper", mapped=False
		)
		self.test_case(
			self.test_case.Boot, self.test_case.Boot.weight, "deep", "deeper", "deepest", mapped=False
		)
		self.assertEqual(len(self.test_case), 7)
		self.assertIn(self.test_case.Boot, self.test_case["deep"])
		self.assertIn(self.test_case.Boot, self.test_case["deep"]["deeper"])
		self.assertIn(self.test_case.Boot, self.test_case["deep"]["deeper"]["deepest"])
		self.assertEqual(
			self.test_case["deep"][self.test_case.Boot], self.test_case.Boot.weight
		)
		self.assertEqual(
			self.test_case["deep"]["deeper"][self.test_case.Boot], self.test_case.Boot.weight
		)
		self.assertEqual(
			self.test_case["deep"]["deeper"]["deepest"][self.test_case.Boot], self.test_case.Boot.weight
		)


		del self.test_case["deep"]
		self.assertEqual(len(self.test_case), 6)




	def test_packed_unload(self):

		self.assertEqual(len(self.test_case), 6)
		self.assertEqual(self.test_case[self.test_case.Diamond],	self.test_case.Diamond.weight)
		self.assertEqual(self.test_case[self.test_case.Ruby],		self.test_case.Ruby.weight)
		self.assertEqual(self.test_case[self.test_case.Jade],		self.test_case.Jade.weight)
		self.assertEqual(self.test_case[self.test_case.Sapphire],	self.test_case.Sapphire.weight)
		self.assertEqual(self.test_case[self.test_case.Gold],		self.test_case.Gold.weight)
		self.assertEqual(self.test_case[self.test_case.Silver],		self.test_case.Silver.weight)

		self.test_case.unload()
		self.assertEqual(len(self.test_case), 0)
		self.assertNotIn(self.test_case.Diamond,	self.test_case)
		self.assertNotIn(self.test_case.Ruby,		self.test_case)
		self.assertNotIn(self.test_case.Jade,		self.test_case)
		self.assertNotIn(self.test_case.Sapphire,	self.test_case)
		self.assertNotIn(self.test_case.Gold,		self.test_case)
		self.assertNotIn(self.test_case.Silver,		self.test_case)
		self.assertNotIn(self.test_case.Boot,		self.test_case)
		self.assertEqual(self.test_case[self.test_case.Diamond],	None)
		self.assertEqual(self.test_case[self.test_case.Ruby],		None)
		self.assertEqual(self.test_case[self.test_case.Jade],		None)
		self.assertEqual(self.test_case[self.test_case.Sapphire],	None)
		self.assertEqual(self.test_case[self.test_case.Gold],		None)
		self.assertEqual(self.test_case[self.test_case.Silver],		None)

		self.test_case(self.test_case.Diamond,	self.test_case.Diamond.weight)
		self.test_case(self.test_case.Ruby,		self.test_case.Ruby.weight)
		self.test_case(self.test_case.Jade,		self.test_case.Jade.weight)
		self.test_case(self.test_case.Sapphire,	self.test_case.Sapphire.weight)
		self.test_case(self.test_case.Gold,		self.test_case.Gold.weight)
		self.test_case(self.test_case.Silver,	self.test_case.Silver.weight)
		self.assertEqual(len(self.test_case), 6)
		self.assertEqual(self.test_case[self.test_case.Diamond],	self.test_case.Diamond.weight)
		self.assertEqual(self.test_case[self.test_case.Ruby],		self.test_case.Ruby.weight)
		self.assertEqual(self.test_case[self.test_case.Jade],		self.test_case.Jade.weight)
		self.assertEqual(self.test_case[self.test_case.Sapphire],	self.test_case.Sapphire.weight)
		self.assertEqual(self.test_case[self.test_case.Gold],		self.test_case.Gold.weight)
		self.assertEqual(self.test_case[self.test_case.Silver],		self.test_case.Silver.weight)




	def test_packed_keysof(self):

		self.assertEqual(

			self.test_case.keysof(self.test_case.Diamond.weight),
			[ self.test_case.Diamond, self.test_case.Ruby ]
		)
		self.assertEqual(

			self.test_case.keysof(self.test_case.Ruby.weight),
			[ self.test_case.Diamond, self.test_case.Ruby ]
		)
		self.assertEqual(self.test_case.keysof(self.test_case.Jade.weight),		[ self.test_case.Jade ])
		self.assertEqual(self.test_case.keysof(self.test_case.Sapphire.weight),	[ self.test_case.Sapphire ])
		self.assertEqual(self.test_case.keysof(self.test_case.Gold.weight),		[ self.test_case.Gold ])
		self.assertEqual(self.test_case.keysof(self.test_case.Silver.weight),	[ self.test_case.Silver ])
		self.assertEqual(self.test_case.keysof(self.test_case.Boot.weight),		[])


		# Broken "_locker_" case
		locker = self.test_case._locker_[:]
		self.test_case._locker_ = []
		self.assertEqual(self.test_case.keysof(self.test_case.Diamond.weight),	[])
		self.assertEqual(self.test_case.keysof(self.test_case.Ruby.weight),		[])
		self.assertEqual(self.test_case.keysof(self.test_case.Jade.weight),		[])
		self.assertEqual(self.test_case.keysof(self.test_case.Sapphire.weight),	[])
		self.assertEqual(self.test_case.keysof(self.test_case.Gold.weight),		[])
		self.assertEqual(self.test_case.keysof(self.test_case.Silver.weight),	[])
		self.assertEqual(self.test_case.keysof(self.test_case.Boot.weight),		[])
		self.test_case._locker_ = locker[:]




	def test_packed_eq_Chest(self):

		self.assertNotEqual(self.test_case, self.not_same_case)
		self.assertEqual(len(self.test_case), len(self.not_same_case))

		self.assertNotEqual(
			self.test_case[self.test_case.Diamond],		self.not_same_case[self.not_same_case.Diamond]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Ruby],		self.not_same_case[self.not_same_case.Ruby]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Jade],		self.not_same_case[self.not_same_case.Jade]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Sapphire],	self.not_same_case[self.not_same_case.Sapphire]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Gold],		self.not_same_case[self.not_same_case.Gold]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Silver],		self.not_same_case[self.not_same_case.Silver]
		)

		self.assertEqual(self.test_case, self.same_case)
		self.assertEqual(len(self.test_case), len(self.same_case))

		self.assertEqual(
			self.test_case[self.test_case.Diamond],		self.same_case[self.test_case.Diamond]
		)
		self.assertEqual(
			self.test_case[self.test_case.Ruby],		self.same_case[self.test_case.Ruby]
		)
		self.assertEqual(
			self.test_case[self.test_case.Jade],		self.same_case[self.test_case.Jade]
		)
		self.assertEqual(
			self.test_case[self.test_case.Sapphire],	self.same_case[self.test_case.Sapphire]
		)
		self.assertEqual(
			self.test_case[self.test_case.Gold],		self.same_case[self.test_case.Gold]
		)
		self.assertEqual(
			self.test_case[self.test_case.Silver],		self.same_case[self.test_case.Silver]
		)




	def test_packed_gt_ge_lt_le_Chest(self):

		self.assertEqual(self.test_case[self.test_case.Diamond],	self.test_case.Diamond.weight)
		self.assertEqual(self.test_case[self.test_case.Ruby],		self.test_case.Ruby.weight)
		self.assertEqual(self.test_case[self.test_case.Jade],		self.test_case.Jade.weight)
		self.assertEqual(self.test_case[self.test_case.Sapphire],	self.test_case.Sapphire.weight)
		self.assertEqual(self.test_case[self.test_case.Gold],		self.test_case.Gold.weight)
		self.assertEqual(self.test_case[self.test_case.Silver],		self.test_case.Silver.weight)
		del self.not_same_case[self.not_same_case.Diamond]

		self.assertNotEqual(self.test_case, self.not_same_case)
		self.assertNotEqual(len(self.test_case), len(self.not_same_case))
		self.assertGreater(self.test_case, self.not_same_case)
		self.assertGreaterEqual(self.test_case, self.not_same_case)
		self.assertLess(self.not_same_case, self.test_case)
		self.assertLessEqual(self.not_same_case, self.test_case)

		self.not_same_case.unload()
		self.not_same_case(self.not_same_case.Diamond,	self.not_same_case.Diamond.weight)
		self.not_same_case(self.not_same_case.Ruby,		self.not_same_case.Ruby.weight)
		self.not_same_case(self.not_same_case.Jade,		self.not_same_case.Jade.weight)
		self.not_same_case(self.not_same_case.Sapphire,	self.not_same_case.Sapphire.weight)
		self.not_same_case(self.not_same_case.Gold,		self.not_same_case.Gold.weight)
		self.not_same_case(self.not_same_case.Silver,	self.not_same_case.Silver.weight)

		self.assertEqual(self.not_same_case[self.not_same_case.Diamond],	self.not_same_case.Diamond.weight)
		self.assertEqual(self.not_same_case[self.not_same_case.Ruby],		self.not_same_case.Ruby.weight)
		self.assertEqual(self.not_same_case[self.not_same_case.Jade],		self.not_same_case.Jade.weight)
		self.assertEqual(self.not_same_case[self.not_same_case.Sapphire],	self.not_same_case.Sapphire.weight)
		self.assertEqual(self.not_same_case[self.not_same_case.Gold],		self.not_same_case.Gold.weight)
		self.assertEqual(self.not_same_case[self.not_same_case.Silver],		self.not_same_case.Silver.weight)




	def test_packed_eq_dict(self):

		self.assertNotEqual(self.test_case, self.not_same_asdict)
		self.assertEqual(len(self.test_case), len(self.not_same_asdict))

		self.assertNotEqual(
			self.test_case[self.test_case.Diamond],		self.not_same_asdict[self.not_same_case.Diamond]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Ruby],		self.not_same_asdict[self.not_same_case.Ruby]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Jade],		self.not_same_asdict[self.not_same_case.Jade]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Sapphire],	self.not_same_asdict[self.not_same_case.Sapphire]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Gold],		self.not_same_asdict[self.not_same_case.Gold]
		)
		self.assertNotEqual(
			self.test_case[self.test_case.Silver],		self.not_same_asdict[self.not_same_case.Silver]
		)


		self.assertEqual(self.test_case, self.same_asdict)
		self.assertEqual(len(self.test_case), len(self.same_asdict))

		self.assertEqual(
			self.test_case[self.test_case.Diamond],		self.same_asdict[self.test_case.Diamond]
		)
		self.assertEqual(
			self.test_case[self.test_case.Ruby],		self.same_asdict[self.test_case.Ruby]
		)
		self.assertEqual(
			self.test_case[self.test_case.Jade],		self.same_asdict[self.test_case.Jade]
		)
		self.assertEqual(
			self.test_case[self.test_case.Sapphire],	self.same_asdict[self.test_case.Sapphire]
		)
		self.assertEqual(
			self.test_case[self.test_case.Gold],		self.same_asdict[self.test_case.Gold]
		)
		self.assertEqual(
			self.test_case[self.test_case.Silver],		self.same_asdict[self.test_case.Silver]
		)




	def test_packed_gt_ge_lt_le_list(self):

		self.assertEqual(self.not_same_asdict[self.not_same_case.Diamond],	self.not_same_case.Diamond.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Ruby],		self.not_same_case.Ruby.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Jade],		self.not_same_case.Jade.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Sapphire],	self.not_same_case.Sapphire.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Gold],		self.not_same_case.Gold.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Silver],	self.not_same_case.Silver.weight)
		del self.not_same_asdict[self.not_same_case.Diamond]

		self.assertNotEqual(self.test_case, self.not_same_asdict)
		self.assertNotEqual(len(self.test_case), len(self.not_same_asdict))
		self.assertGreater(self.test_case, self.not_same_asdict)
		self.assertGreaterEqual(self.test_case, self.not_same_asdict)
		self.assertLess(self.not_same_asdict, self.test_case)
		self.assertLessEqual(self.not_same_asdict, self.test_case)

		del self.not_same_asdict[self.not_same_case.Ruby]
		del self.not_same_asdict[self.not_same_case.Jade]
		del self.not_same_asdict[self.not_same_case.Sapphire]
		del self.not_same_asdict[self.not_same_case.Gold]
		del self.not_same_asdict[self.not_same_case.Silver]
		self.assertFalse(self.not_same_asdict)
		self.not_same_asdict[self.not_same_case.Diamond]	=	self.not_same_case.Diamond.weight
		self.not_same_asdict[self.not_same_case.Ruby]		=	self.not_same_case.Ruby.weight
		self.not_same_asdict[self.not_same_case.Jade]		=	self.not_same_case.Jade.weight
		self.not_same_asdict[self.not_same_case.Sapphire]	=	self.not_same_case.Sapphire.weight
		self.not_same_asdict[self.not_same_case.Gold]		=	self.not_same_case.Gold.weight
		self.not_same_asdict[self.not_same_case.Silver]		=	self.not_same_case.Silver.weight

		self.assertEqual(self.not_same_asdict[self.not_same_case.Diamond],	self.not_same_case.Diamond.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Ruby],		self.not_same_case.Ruby.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Jade],		self.not_same_case.Jade.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Sapphire],	self.not_same_case.Sapphire.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Gold],		self.not_same_case.Gold.weight)
		self.assertEqual(self.not_same_asdict[self.not_same_case.Silver],	self.not_same_case.Silver.weight)




	def test_packed_iter(self):

		count = 0
		for K,V in self.test_case:

			self.assertEqual(K.weight, V)
			self.assertIsInstance(K(), str)
			count += 1

		self.assertEqual(count, len(self.test_case))
		count = 0
		_case = iter(self.test_case)
		for one,two in zip(_case, _case):

			Kone,Vone = one
			self.assertEqual(Kone.weight, Vone)
			self.assertIsInstance(Kone(), str)
			count += 1
			Ktwo,Vtwo = two
			self.assertEqual(Ktwo.weight, Vtwo)
			self.assertIsInstance(Ktwo(), str)
			count += 1

		self.assertEqual(count, len(self.test_case))


		# Breaking iteration by messing with "_locker_":
		locker = self.test_case._locker_[:]
		self.test_case._locker_ = []
		count = 0
		for K,V in self.test_case:

			self.assertEqual(K.weight, V)
			self.assertIsInstance(K(), str)
			count += 1

		self.assertEqual(count, 0)
		self.assertEqual(len(self.test_case), 6)
		self.assertNotEqual(count, len(self.test_case))
		self.test_case._locker_ = locker[:]




	def test_packed_next(self):

		count = 0
		for _ in range(3 *len(self.test_case)):

			try:

				K,V = next(self.test_case)
				self.assertEqual(K.weight, V)
				self.assertIsInstance(K(), str)
				count += 1

			except	StopIteration:

				self.assertEqual(count, len(self.test_case))
				count = 0


		# Breaking "_locker_" and __next__ always raises StopIteration
		locker = self.test_case._locker_[:]
		self.test_case._locker_ = []
		count = 0
		for _ in range(3):

			try:

				next(self.test_case)
				count += 1

			except	StopIteration : self.assertEqual(count,0)

		self.assertEqual(count,0)
		self.test_case._locker_ = locker[:]




	def test_packed_reversed(self):

		# Direct
		self.assertEqual(
			[ V for K,V in self.test_case ],
			[
				self.test_case.Diamond.weight,
				self.test_case.Ruby.weight,
				self.test_case.Jade.weight,
				self.test_case.Sapphire.weight,
				self.test_case.Gold.weight,
				self.test_case.Silver.weight
			]
		)

		# Reversed
		self.assertEqual(
			[ V for K,V in reversed(self.test_case) ],
			[
				self.test_case.Silver.weight,
				self.test_case.Gold.weight,
				self.test_case.Sapphire.weight,
				self.test_case.Jade.weight,
				self.test_case.Ruby.weight,
				self.test_case.Diamond.weight
			]
		)

		# Broken "_locker_" that already reversed
		locker = self.test_case._locker_[:]
		self.test_case._locker_ = list(reversed(self.test_case._locker_))
		self.assertEqual(
			[ V for K,V in reversed(self.test_case) ],
			[
				self.test_case.Diamond.weight,
				self.test_case.Ruby.weight,
				self.test_case.Jade.weight,
				self.test_case.Sapphire.weight,
				self.test_case.Gold.weight,
				self.test_case.Silver.weight
			]
		)

		# Broken "_locker_" that is empty
		self.test_case._locker_ = []
		self.assertEqual([ V for K,V in self.test_case ], [])
		self.test_case._locker_ = locker[:]




	def test_packed_noload_call(self):

		test_case = self.test_case()
		self.assertIsInstance(test_case, dict)
		self.assertEqual(test_case, self.test_case())
		self.assertNotEqual(id(test_case), id(self.test_case))

		self.assertEqual(self.test_case[self.test_case.Diamond],	test_case[str(self.test_case.Diamond)])
		self.assertEqual(self.test_case[self.test_case.Ruby],		test_case[str(self.test_case.Ruby)])
		self.assertEqual(self.test_case[self.test_case.Jade],		test_case[str(self.test_case.Jade)])
		self.assertEqual(self.test_case[self.test_case.Sapphire],	test_case[str(self.test_case.Sapphire)])
		self.assertEqual(self.test_case[self.test_case.Gold],		test_case[str(self.test_case.Gold)])
		self.assertEqual(self.test_case[self.test_case.Silver],		test_case[str(self.test_case.Silver)])
















class DeepChest(MagicalTestCase):

	"""
		Testing deep method.
	"""

	@classmethod
	def tearDownClass(cls):

		if	cls.clean_up:
			if	os.path.isfile(cls.DEEP_KEY_SET_HANDLER): os.remove(cls.DEEP_KEY_SET_HANDLER)

	@classmethod
	def setUpClass(cls):
		class Digituts(KeyChest):

			class loggy(LibraryContrib):

				init_name	= "deep"
				init_level	= 10
				handler		= cls.DEEP_KEY_SET_HANDLER


		cls.make_loggy_file(cls, cls.DEEP_KEY_SET_HANDLER)
		cls.test_case = Digituts()
		cls.test_case(10, "the ten", *range(10), mapped=False)


	def test_consistency(self):

		self.assertEqual(len(self.test_case),1)

		self.assertIn(0, self.test_case)
		self.assertIsInstance(self.test_case[0], dict)

		self.assertIn(1, self.test_case[0])
		self.assertIsInstance(self.test_case[0][1], dict)

		self.assertIn(2, self.test_case[0][1])
		self.assertIsInstance(self.test_case[0][1][2], dict)

		self.assertIn(3, self.test_case[0][1][2])
		self.assertIsInstance(self.test_case[0][1][2][3], dict)

		self.assertIn(4, self.test_case[0][1][2][3])
		self.assertIsInstance(self.test_case[0][1][2][3][4], dict)

		self.assertIn(5, self.test_case[0][1][2][3][4])
		self.assertIsInstance(self.test_case[0][1][2][3][4][5], dict)

		self.assertIn(6, self.test_case[0][1][2][3][4][5])
		self.assertIsInstance(self.test_case[0][1][2][3][4][5][6], dict)

		self.assertIn(7, self.test_case[0][1][2][3][4][5][6])
		self.assertIsInstance(self.test_case[0][1][2][3][4][5][6][7], dict)

		self.assertIn(8, self.test_case[0][1][2][3][4][5][6][7])
		self.assertIsInstance(self.test_case[0][1][2][3][4][5][6][7][8], dict)

		self.assertIn(9, self.test_case[0][1][2][3][4][5][6][7][8])
		self.assertIsInstance(self.test_case[0][1][2][3][4][5][6][7][8][9], dict)

		self.assertIn(10, self.test_case[0][1][2][3][4][5][6][7][8][9])
		self.assertIsInstance(self.test_case[0][1][2][3][4][5][6][7][8][9][10], str)
		self.assertEqual(self.test_case[0][1][2][3][4][5][6][7][8][9][10], "the ten")


	def test_deep_1(self):

		for i in range(2,10):	self.assertIsInstance(self.test_case.deep(*range(i)), dict)


	def test_deep_2(self):

		self.assertEqual(self.test_case.deep(*range(1)), self.test_case[0])
		self.assertEqual(self.test_case.deep(*range(2)), self.test_case[0][1])
		self.assertEqual(self.test_case.deep(*range(3)), self.test_case[0][1][2])
		self.assertEqual(self.test_case.deep(*range(4)), self.test_case[0][1][2][3])
		self.assertEqual(self.test_case.deep(*range(5)), self.test_case[0][1][2][3][4])
		self.assertEqual(self.test_case.deep(*range(6)), self.test_case[0][1][2][3][4][5])
		self.assertEqual(self.test_case.deep(*range(7)), self.test_case[0][1][2][3][4][5][6])
		self.assertEqual(self.test_case.deep(*range(8)), self.test_case[0][1][2][3][4][5][6][7])
		self.assertEqual(self.test_case.deep(*range(9)), self.test_case[0][1][2][3][4][5][6][7][8])
		self.assertEqual(self.test_case.deep(*range(10)), self.test_case[0][1][2][3][4][5][6][7][8][9])
		self.assertEqual(self.test_case.deep(*range(11)), "the ten")


	def test_deep_3(self):

		self.assertIsNone(self.test_case.deep(*range(1), 42))
		self.assertIsNone(self.test_case.deep(*range(2), 42))
		self.assertIsNone(self.test_case.deep(*range(3), 42))
		self.assertIsNone(self.test_case.deep(*range(4), 42))
		self.assertIsNone(self.test_case.deep(*range(5), 42))
		self.assertIsNone(self.test_case.deep(*range(6), 42))
		self.assertIsNone(self.test_case.deep(*range(7), 42))
		self.assertIsNone(self.test_case.deep(*range(8), 42))
		self.assertIsNone(self.test_case.deep(*range(9), 42))
		self.assertIsNone(self.test_case.deep(*range(10), 42))


	def test_deep_4(self):

		for j in range(10, 1, -1):
			with self.assertLogs("deep", 10) as case_loggy:

				self.assertIsNone(self.test_case.deep(j))
				self.assertIn(f"DEBUG:deep:Root-key \"{j}\" not found", case_loggy.output)
				self.assertIsNone(self.test_case.deep(*range(1,j)))
				self.assertIn(f"DEBUG:deep:Root-key \"1\" not found", case_loggy.output)


	def test_deep_5(self):

		with self.assertLogs("deep", 10) as case_loggy:

			self.assertIsNone(self.test_case.deep())
			self.assertIn(f"DEBUG:deep:No depth to go deep", case_loggy.output)








if	__name__ == "__main__" : unittest.main(verbosity=2)










import	os
import	unittest
from	time								import sleep
from	pygwarts.tests.hagrid				import EasySet
from	pygwarts.irma.contrib				import LibraryContrib
from	pygwarts.hagrid.thrivables			import Tree
from	pygwarts.hagrid.sprouts				import fssprout
from	pygwarts.hagrid.planting			import Flourish
from	pygwarts.hagrid.bloom.twigs			import Germination
from	pygwarts.hagrid.bloom.leafs			import Transfer
from	pygwarts.hagrid.planting.leafs		import LeafMove
from	pygwarts.hagrid.planting.twigs		import TwigThrive
from	pygwarts.hagrid.planting.peels		import ThrivingPeel
from	pygwarts.hagrid.cultivation.sifting	import SiftingController








class TransferCase(EasySet):

	"""
		ThrivingPeel, Germination -> Transfer
		Single sprout
		Single bough
	"""

	@classmethod
	def tearDownClass(cls):
		cls.clean(cls)

		if	cls.clean_up:
			if	os.path.isfile(cls.EASY_HANDLER_4): os.remove(cls.EASY_HANDLER_4)

	@classmethod
	def setUpClass(cls):

		super().setUpClass()
		cls.make_loggy_file(cls, cls.EASY_HANDLER_4)


	def setUp(self):
		class Sakura(Tree):

			bough = self.EASY_SET_BOUGH
			class loggy(LibraryContrib):

				handler		= self.EASY_HANDLER_4
				init_level	= 10

			@ThrivingPeel("today", "records", to_peak=False)
			class thrive(TwigThrive):	pass
			class folders(Germination):	pass

			@ThrivingPeel("today", "records", to_peak=False)
			class graft(LeafMove):		pass
			class files(Transfer):		pass

			@fssprout(self.EASY_SET_SPROUT)
			class sync(Flourish):

				class twigs(SiftingController):

					include	= (

						( os.path.join(self.EASY_SET_SPROUT, "pros"), )
						if os.name == "posix" else
						( os.path.join(self.EASY_SET_SPROUT, "pros").replace("\\", "\\\\"), )
					)
					exclude	= (

						( os.path.join(self.EASY_SET_SPROUT, "pros", ".+"), )
						if os.name == "posix" else
						( os.path.join(self.EASY_SET_SPROUT, "pros", ".+").replace("\\", "\\\\"), )
					)

				class leafs(SiftingController):

					include	= r".+\.txt$",
					exclude	= (

						( rf".+/[^/]+good.*\.txt$", )
						if os.name == "posix" else
						( rf".+\\[^(\\)]+good.*\.txt$", )
					)

		self.test_case = Sakura


	def fdthriving(self, dst :str) -> str :

		"""
			Makes destination file/folder path according to hardcoded thriving
		"""

		return	os.path.join(
			self.EASY_SET_BOUGH, "today", "records", os.path.relpath(dst, self.EASY_SET_BOUGH)
		)




	def test_first_flourish(self):

		self.test_case.loggy.init_name = "first_flourish"
		self.assertTrue(os.path.isdir(self.EASY_SET_BOUGH))
		with self.assertLogs("first_flourish", 10) as case_loggy:

			self.test_case = self.test_case()
			self.test_case.sync()


		self.no_loggy_levels(case_loggy.output, 30,40,50)


		# Included file "WitchDoctor.txt"
		self.assertTrue(os.path.isfile(self.fdthriving(self.dst_file1)))
		# Included folder "pros"
		self.assertTrue(os.path.isdir(self.fdthriving(self.dst_pros_folder)))
		# File excluded by ".+{os.sep}[^{os.sep}]+good.*\.txt$"
		self.assertFalse(os.path.isfile(self.fdthriving(self.dst_pros_file1)))
		# Files included by ".+\.txt$"
		self.assertTrue(os.path.isfile(self.fdthriving(self.dst_pros_file2)))
		self.assertTrue(os.path.isfile(self.fdthriving(self.dst_pros_file3)))
		# Folders and files sifted out by being not included
		self.assertFalse(os.path.isdir(self.fdthriving(self.dst_cons_folder)))
		self.assertFalse(os.path.isfile(self.fdthriving(self.dst_cons_file1)))
		self.assertFalse(os.path.isfile(self.fdthriving(self.dst_cons_file2)))
		self.assertFalse(os.path.isdir(self.fdthriving(self.dst_redundant_1_folder)))
		self.assertFalse(os.path.isdir(self.fdthriving(self.dst_redundant_2_folder)))
		self.assertFalse(os.path.isfile(self.fdthriving(self.dst_redundant_1)))
		self.assertFalse(os.path.isfile(self.fdthriving(self.dst_redundant_2)))

		self.assertFalse(os.path.isfile(self.file1))
		self.assertTrue(os.path.isfile(self.pros_file1))
		self.assertFalse(os.path.isfile(self.pros_file2))
		self.assertFalse(os.path.isfile(self.pros_file3))
		self.assertTrue(os.path.isfile(self.cons_file1))
		self.assertTrue(os.path.isfile(self.cons_file2))
		self.assertTrue(os.path.isfile(self.redundant_1))
		self.assertTrue(os.path.isfile(self.redundant_2))

		self.assertIn(

			f"INFO:first_flourish:Moved leaf \"{self.fdthriving(self.dst_file1)}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Thrived twig \"{self.fdthriving(self.dst_pros_folder)}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Moved leaf \"{self.fdthriving(self.dst_pros_file2)}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Moved leaf \"{self.fdthriving(self.dst_pros_file3)}\"",
			case_loggy.output
		)
		self.assertIn(f"DEBUG:first_flourish:Sifted sprig \"cons\"", case_loggy.output)
		self.assertIn(f"DEBUG:first_flourish:Sifted sprig \"not so good.txt\"", case_loggy.output)
		self.assertIn(f"DEBUG:first_flourish:Sifted sprig \"redundant_folder_1\"", case_loggy.output)
		self.assertIn(f"DEBUG:first_flourish:Sifted sprig \"redundant_folder_2\"", case_loggy.output)
		self.assertNotIn(

			f"INFO:first_flourish:Moved leaf \"{self.fdthriving(self.dst_pros_file1)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:first_flourish:Moved leaf \"{self.fdthriving(self.dst_cons_file1)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:first_flourish:Moved leaf \"{self.fdthriving(self.dst_cons_file2)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:first_flourish:Moved leaf \"{self.fdthriving(self.dst_redundant_1)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:first_flourish:Moved leaf \"{self.fdthriving(self.dst_redundant_2)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:first_flourish:Thrived twig \"{self.fdthriving(self.dst_cons_folder)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:first_flourish:Thrived twig \"{self.fdthriving(self.dst_redundant_1_folder)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:first_flourish:Thrived twig \"{self.fdthriving(self.dst_redundant_2_folder)}\"",
			case_loggy.output
		)








	def test_second_flourish(self):

		sleep(1.1)
		self.test_case.loggy.init_name = "second_flourish"
		with self.assertLogs("second_flourish", 10) as case_loggy:

			self.test_case = self.test_case()
			self.test_case.sync()


		self.no_loggy_levels(case_loggy.output, 30,40,50)


		# Included file "WitchDoctor.txt"
		self.assertTrue(os.path.isfile(self.fdthriving(self.dst_file1)))
		# Included folder "pros"
		self.assertTrue(os.path.isdir(self.fdthriving(self.dst_pros_folder)))
		# File excluded by ".+{os.sep}[^{os.sep}]+good.*\.txt$"
		self.assertFalse(os.path.isfile(self.fdthriving(self.dst_pros_file1)))
		# Files included by ".+\.txt$"
		self.assertTrue(os.path.isfile(self.fdthriving(self.dst_pros_file2)))
		self.assertTrue(os.path.isfile(self.fdthriving(self.dst_pros_file3)))
		# Folders and files sifted out by being not included
		self.assertFalse(os.path.isdir(self.fdthriving(self.dst_cons_folder)))
		self.assertFalse(os.path.isfile(self.fdthriving(self.dst_cons_file1)))
		self.assertFalse(os.path.isfile(self.fdthriving(self.dst_cons_file2)))
		self.assertFalse(os.path.isdir(self.fdthriving(self.dst_redundant_1_folder)))
		self.assertFalse(os.path.isdir(self.fdthriving(self.dst_redundant_2_folder)))
		self.assertFalse(os.path.isfile(self.fdthriving(self.dst_redundant_1)))
		self.assertFalse(os.path.isfile(self.fdthriving(self.dst_redundant_2)))

		self.assertFalse(os.path.isfile(self.file1))
		self.assertTrue(os.path.isfile(self.pros_file1))
		self.assertFalse(os.path.isfile(self.pros_file2))
		self.assertFalse(os.path.isfile(self.pros_file3))
		self.assertTrue(os.path.isfile(self.cons_file1))
		self.assertTrue(os.path.isfile(self.cons_file2))
		self.assertTrue(os.path.isfile(self.redundant_1))
		self.assertTrue(os.path.isfile(self.redundant_2))

		self.assertNotIn(

			f"INFO:second_flourish:Moved leaf \"{self.fdthriving(self.dst_file1)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:second_flourish:Thrived twig \"{self.fdthriving(self.dst_pros_folder)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:second_flourish:Moved leaf \"{self.fdthriving(self.dst_pros_file2)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:second_flourish:Moved leaf \"{self.fdthriving(self.dst_pros_file3)}\"",
			case_loggy.output
		)
		self.assertIn(f"DEBUG:second_flourish:Sifted sprig \"cons\"", case_loggy.output)
		self.assertIn(f"DEBUG:second_flourish:Sifted sprig \"not so good.txt\"", case_loggy.output)
		self.assertIn(f"DEBUG:second_flourish:Sifted sprig \"redundant_folder_1\"", case_loggy.output)
		self.assertIn(f"DEBUG:second_flourish:Sifted sprig \"redundant_folder_2\"", case_loggy.output)
		self.assertNotIn(

			f"INFO:second_flourish:Moved leaf \"{self.fdthriving(self.dst_pros_file1)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:second_flourish:Moved leaf \"{self.fdthriving(self.dst_cons_file1)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:second_flourish:Moved leaf \"{self.fdthriving(self.dst_cons_file2)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:second_flourish:Moved leaf \"{self.fdthriving(self.dst_redundant_1)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:second_flourish:Moved leaf \"{self.fdthriving(self.dst_redundant_2)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:second_flourish:Thrived twig \"{self.fdthriving(self.dst_cons_folder)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:second_flourish:Thrived twig \"{self.fdthriving(self.dst_redundant_1_folder)}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:second_flourish:Thrived twig \"{self.fdthriving(self.dst_redundant_2_folder)}\"",
			case_loggy.output
		)








if	__name__ == "__main__" : unittest.main(verbosity=2)










import	os
import	unittest
from	time								import sleep
from	pygwarts.tests.hagrid				import MediumSet
from	pygwarts.irma.contrib				import LibraryContrib
from	pygwarts.hagrid.thrivables			import Tree
from	pygwarts.hagrid.thrivables			import Copse
from	pygwarts.hagrid.sprouts				import fssprout
from	pygwarts.hagrid.planting			import Flourish
from	pygwarts.hagrid.bloom.weeds			import Efflorescence
from	pygwarts.hagrid.cultivation.sifting	import SiftingController
from	pygwarts.hagrid.planting.weeds		import SprigTrimmer








class MultiBoughEfflorescence(MediumSet):

	"""
		Efflorescence
		Single sprout copse
		Triple bough copse
	"""

	@classmethod
	def tearDownClass(cls):
		cls.clean(cls)

		if	cls.clean_up:
			if	os.path.isfile(cls.MEDIUM_HANDLER_5): os.remove(cls.MEDIUM_HANDLER_5)

	@classmethod
	def setUpClass(cls):

		"""
			d- MEDIUM_SET_BOUGH_1
				d- men due
					f- rock
					f- paper
					f- scissors
					f- lizzard						- this one to be effloresced
					f- spock						- this one to be effloresced
					d- girls cries
						f- sad movies
						f- dumbasses (literaly)
						f- onion
						d- mirror reflections		- this one to be effloresced
				d- commons
					d- good
					d- not good
					d- bad
					d- not bad
					d- not not
					d- bad bad
					d- bad good
					d- good bad
					d- big bad wolf					- this one to be effloresced
		"""

		super().setUpClass()
		cls.make_loggy_file(cls, cls.MEDIUM_HANDLER_5)


		# First bough first sprout
		cls.fmake(cls, cls.tg1_f_mss1_md_rock, "it's about time it's about power")
		cls.fmake(cls, cls.tg1_f_mss1_md_paper, "if cut it's like the face insight is right beneath my skin")
		cls.fmake(cls, cls.tg1_f_mss1_md_scissors, "only in HD or better")
		cls.fmake(cls, cls.tg1_f_mss1_md_gc_sadmovies, "green mile, knockin' on heaven's door")
		cls.fmake(cls, cls.tg1_f_mss1_md_gc_dumbasses, "HA-HA, CLASSIC")
		cls.fmake(cls, cls.tg1_f_mss1_md_gc_onion, "makes even devil cry")
		os.makedirs(cls.tg1_d_mss1_c_good)
		os.makedirs(cls.tg1_d_mss1_c_notgood)
		os.makedirs(cls.tg1_d_mss1_c_bad)
		os.makedirs(cls.tg1_d_mss1_c_notbad)
		os.makedirs(cls.tg1_d_mss1_c_notnot)
		os.makedirs(cls.tg1_d_mss1_c_badbad)
		os.makedirs(cls.tg1_d_mss1_c_badgood)
		os.makedirs(cls.tg1_d_mss1_c_goodbad)


		# Second bough first sprout
		cls.fmake(cls, cls.tg2_f_mss1_md_rock, "it's about time it's about power")
		cls.fmake(cls, cls.tg2_f_mss1_md_paper, "if cut it's like the face insight is right beneath my skin")
		cls.fmake(cls, cls.tg2_f_mss1_md_scissors, "only in HD or better")
		cls.fmake(cls, cls.tg2_f_mss1_md_gc_sadmovies, "green mile, knockin' on heaven's door")
		cls.fmake(cls, cls.tg2_f_mss1_md_gc_dumbasses, "HA-HA, CLASSIC")
		cls.fmake(cls, cls.tg2_f_mss1_md_gc_onion, "makes even devil cry")
		os.makedirs(cls.tg2_d_mss1_c_good)
		os.makedirs(cls.tg2_d_mss1_c_notgood)
		os.makedirs(cls.tg2_d_mss1_c_bad)
		os.makedirs(cls.tg2_d_mss1_c_notbad)
		os.makedirs(cls.tg2_d_mss1_c_notnot)
		os.makedirs(cls.tg2_d_mss1_c_badbad)
		os.makedirs(cls.tg2_d_mss1_c_badgood)
		os.makedirs(cls.tg2_d_mss1_c_goodbad)


		# Third bough first sprout
		cls.fmake(cls, cls.tg3_f_mss1_md_rock, "it's about time it's about power")
		cls.fmake(cls, cls.tg3_f_mss1_md_paper, "if cut it's like the face insight is right beneath my skin")
		cls.fmake(cls, cls.tg3_f_mss1_md_scissors, "only in HD or better")
		cls.fmake(cls, cls.tg3_f_mss1_md_gc_sadmovies, "green mile, knockin' on heaven's door")
		cls.fmake(cls, cls.tg3_f_mss1_md_gc_dumbasses, "HA-HA, CLASSIC")
		cls.fmake(cls, cls.tg3_f_mss1_md_gc_onion, "makes even devil cry")
		os.makedirs(cls.tg3_d_mss1_c_good)
		os.makedirs(cls.tg3_d_mss1_c_notgood)
		os.makedirs(cls.tg3_d_mss1_c_bad)
		os.makedirs(cls.tg3_d_mss1_c_notbad)
		os.makedirs(cls.tg3_d_mss1_c_notnot)
		os.makedirs(cls.tg3_d_mss1_c_badbad)
		os.makedirs(cls.tg3_d_mss1_c_badgood)
		os.makedirs(cls.tg3_d_mss1_c_goodbad)


		# First bough extra
		cls.tg1_f_mss1_md_lizzard	= os.path.join(cls.tg1_d_mss1_md, "lizzard")
		cls.tg1_f_mss1_md_spock		= os.path.join(cls.tg1_d_mss1_md, "spock")
		cls.tg1_d_mss1_md_d_mirror	= os.path.join(cls.tg1_d_mss1_md_gc, "mirror reflections")
		cls.tg1_d_mss1_c_bbw		= os.path.join(cls.tg1_d_mss1_c, "big bad wolf")
		cls.fmake(cls, cls.tg1_f_mss1_md_lizzard, "poisones spock")
		cls.fmake(cls, cls.tg1_f_mss1_md_spock, "vaporizes rock")
		os.makedirs(cls.tg1_d_mss1_md_d_mirror)
		os.makedirs(cls.tg1_d_mss1_c_bbw)


		# Second bough extra
		cls.tg2_f_mss1_md_lizzard	= os.path.join(cls.tg2_d_mss1_md, "lizzard")
		cls.tg2_f_mss1_md_spock		= os.path.join(cls.tg2_d_mss1_md, "spock")
		cls.tg2_d_mss1_md_d_mirror	= os.path.join(cls.tg2_d_mss1_md_gc, "mirror reflections")
		cls.tg2_d_mss1_c_bbw		= os.path.join(cls.tg2_d_mss1_c, "big bad wolf")
		cls.fmake(cls, cls.tg2_f_mss1_md_lizzard, "poisones spock")
		cls.fmake(cls, cls.tg2_f_mss1_md_spock, "vaporizes rock")
		os.makedirs(cls.tg2_d_mss1_md_d_mirror)
		os.makedirs(cls.tg2_d_mss1_c_bbw)


		# Third bough extra
		cls.tg3_f_mss1_md_lizzard	= os.path.join(cls.tg3_d_mss1_md, "lizzard")
		cls.tg3_f_mss1_md_spock		= os.path.join(cls.tg3_d_mss1_md, "spock")
		cls.tg3_d_mss1_md_d_mirror	= os.path.join(cls.tg3_d_mss1_md_gc, "mirror reflections")
		cls.tg3_d_mss1_c_bbw		= os.path.join(cls.tg3_d_mss1_c, "big bad wolf")
		cls.fmake(cls, cls.tg3_f_mss1_md_lizzard, "poisones spock")
		cls.fmake(cls, cls.tg3_f_mss1_md_spock, "vaporizes rock")
		os.makedirs(cls.tg3_d_mss1_md_d_mirror)
		os.makedirs(cls.tg3_d_mss1_c_bbw)


	def setUp(self):
		class Forest(Copse):

			class loggy(LibraryContrib):

				handler		= self.MEDIUM_HANDLER_5
				init_level	= 10

			class First(Tree):

				bough = self.MEDIUM_SET_BOUGH_1
				class leafs(SiftingController):

					exclude = ( r".+/lizzard", ) if os.name == "posix" else ( r".+\\lizzard", )
					include = (
						(
							rf"{self.MEDIUM_SET_BOUGH_1}/.+",
							rf"{self.MEDIUM_SET_SPROUT_1}/.+",
						)	if os.name == "posix" else
						(
							self.MEDIUM_SET_BOUGH_1.replace("\\", "\\\\") + r"\\.+",
							self.MEDIUM_SET_SPROUT_1.replace("\\", "\\\\") + r"\\.+",
						)
					)

				class twigs(SiftingController):

					exclude = ( r".+/mirror.+", ) if os.name == "posix" else ( r".+\\mirror.+", )
					include = (
						(
							rf"{self.MEDIUM_SET_BOUGH_1}/.+",
							rf"{self.MEDIUM_SET_SPROUT_1}/.+",
						)	if os.name == "posix" else
						(
							self.MEDIUM_SET_BOUGH_1.replace("\\", "\\\\") + r"\\.+",
							self.MEDIUM_SET_SPROUT_1.replace("\\", "\\\\") + r"\\.+",
						)
					)

			class Second(Tree):

				bough = self.MEDIUM_SET_BOUGH_2
				class leafs(SiftingController):

					exclude = ( r".+/spock", ) if os.name == "posix" else ( r".+\\spock", )
					include = (
						(
							rf"{self.MEDIUM_SET_BOUGH_2}/.+",
							rf"{self.MEDIUM_SET_SPROUT_1}/.+",
						)	if os.name == "posix" else
						(
							self.MEDIUM_SET_BOUGH_2.replace("\\", "\\\\") + r"\\.+",
							self.MEDIUM_SET_SPROUT_1.replace("\\", "\\\\") + r"\\.+",
						)
					)

				class twigs(SiftingController):

					exclude = ( r".+/big.+", ) if os.name == "posix" else ( r".+\\big.+", )
					include = (
						(
							rf"{self.MEDIUM_SET_BOUGH_2}/.+",
							rf"{self.MEDIUM_SET_SPROUT_1}/.+",
						)	if os.name == "posix" else
						(
							self.MEDIUM_SET_BOUGH_2.replace("\\", "\\\\") + r"\\.+",
							self.MEDIUM_SET_SPROUT_1.replace("\\", "\\\\") + r"\\.+",
						)
					)

			class Third(Tree):

				bough = self.MEDIUM_SET_BOUGH_3
				class leafs(SiftingController):

					include = (
						(
							rf"{self.MEDIUM_SET_BOUGH_3}/.+",
							rf"{self.MEDIUM_SET_SPROUT_1}/.+",
						)	if os.name == "posix" else
						(
							self.MEDIUM_SET_BOUGH_3.replace("\\", "\\\\") + r"\\.+",
							self.MEDIUM_SET_SPROUT_1.replace("\\", "\\\\") + r"\\.+",
						)
					)

				class twigs(SiftingController):

					include = (
						(
							rf"{self.MEDIUM_SET_BOUGH_3}/.+",
							rf"{self.MEDIUM_SET_SPROUT_1}/.+",
						)	if os.name == "posix" else
						(
							self.MEDIUM_SET_BOUGH_3.replace("\\", "\\\\") + r"\\.+",
							self.MEDIUM_SET_SPROUT_1.replace("\\", "\\\\") + r"\\.+",
						)
					)

			class trim(SprigTrimmer):	pass
			class clean(Efflorescence):
				branches	= {

					self.MEDIUM_SET_BOUGH_1: ( self.MEDIUM_SET_SPROUT_1, ),
					self.MEDIUM_SET_BOUGH_2: ( self.MEDIUM_SET_SPROUT_1, ),
					self.MEDIUM_SET_BOUGH_3: ( self.MEDIUM_SET_SPROUT_1, ),
				}


			@fssprout(self.MEDIUM_SET_SPROUT_1)
			class sync(Flourish):		pass


		self.test_case = Forest




	def test_first_flourish(self):

		self.test_case.loggy.init_name = "first_flourish"
		self.assertTrue(os.path.isdir(self.MEDIUM_SET_BOUGH_1))


		# First sprout
		self.assertTrue(os.path.isfile(self.f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))


		# First sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_goodbad))
		# First bough extra
		self.assertTrue(os.path.isfile(self.f_msb1_flower))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_lizzard))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_spock))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_d_mirror))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_bbw))


		# First sprout second bough
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_goodbad))
		# Second bough extra
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_lizzard))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_spock))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md_d_mirror))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bbw))
		self.assertTrue(os.path.isdir(self.d_msb2_w))


		# First sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_goodbad))
		# Third bough extra
		self.assertTrue(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_lizzard))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_spock))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md_d_mirror))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_bbw))
		self.assertTrue(os.path.isdir(self.d_msb3_a))
		self.assertTrue(os.path.isdir(self.d_msb3_t))




		with self.assertLogs("first_flourish", 10) as case_loggy:

			self.test_case = self.test_case()
			self.test_case.sync()




		self.no_loggy_levels(case_loggy.output, 30,40,50)


		self.assertIn(

			f"INFO:first_flourish:Trimmed twig \"{self.tg1_d_mss1_c_bbw}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Trimmed leaf \"{self.tg1_f_mss1_md_spock}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Trimmed leaf \"{self.f_msb1_flower}\"",
			case_loggy.output
		)





		self.assertIn(

			f"INFO:first_flourish:Trimmed twig \"{self.d_msb2_w}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Trimmed twig \"{self.tg2_d_mss1_md_d_mirror}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Trimmed leaf \"{self.tg2_f_mss1_md_lizzard}\"",
			case_loggy.output
		)




		self.assertIn(

			f"INFO:first_flourish:Trimmed twig \"{self.tg3_d_mss1_md_d_mirror}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Trimmed twig \"{self.tg3_d_mss1_c_bbw}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Trimmed twig \"{self.d_msb3_a}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Trimmed twig \"{self.d_msb3_t}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Trimmed leaf \"{self.f_msb3_almost}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Trimmed leaf \"{self.tg3_f_mss1_md_lizzard}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:first_flourish:Trimmed leaf \"{self.tg3_f_mss1_md_spock}\"",
			case_loggy.output
		)




		# First sprout
		self.assertTrue(os.path.isfile(self.f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))


		# First sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_goodbad))
		# First bough extra
		self.assertFalse(os.path.isfile(self.f_msb1_flower))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_lizzard))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_spock))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_d_mirror))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_bbw))


		# First sprout second bough
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_goodbad))
		# Second bough extra
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_lizzard))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_spock))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_md_d_mirror))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bbw))
		self.assertFalse(os.path.isdir(self.d_msb2_w))


		# First sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_goodbad))
		# Third bough extra
		self.assertFalse(os.path.isfile(self.f_msb3_almost))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_lizzard))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_spock))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_md_d_mirror))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_bbw))
		self.assertFalse(os.path.isdir(self.d_msb3_a))
		self.assertFalse(os.path.isdir(self.d_msb3_t))
















	def test_no_touch_flourish(self):

		sleep(1.1)
		self.test_case.loggy.init_name = "no_touch_flourish"


		# First sprout
		self.assertTrue(os.path.isfile(self.f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))


		# First sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_goodbad))
		# First bough extra
		self.assertFalse(os.path.isfile(self.f_msb1_flower))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_lizzard))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_spock))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_d_mirror))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_bbw))


		# First sprout second bough
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_goodbad))
		# Second bough extra
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_lizzard))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_spock))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_md_d_mirror))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bbw))
		self.assertFalse(os.path.isdir(self.d_msb2_w))


		# First sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_goodbad))
		# Third bough extra
		self.assertFalse(os.path.isfile(self.f_msb3_almost))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_lizzard))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_spock))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_md_d_mirror))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_bbw))
		self.assertFalse(os.path.isdir(self.d_msb3_a))
		self.assertFalse(os.path.isdir(self.d_msb3_t))




		with self.assertLogs("no_touch_flourish", 10) as case_loggy:

			self.test_case = self.test_case()
			self.test_case.sync()




		self.no_loggy_levels(case_loggy.output, 30,40,50)


		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed twig \"{self.tg1_d_mss1_c_bbw}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed leaf \"{self.tg1_f_mss1_md_spock}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed leaf \"{self.f_msb1_flower}\"",
			case_loggy.output
		)





		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed twig \"{self.d_msb2_w}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed twig \"{self.tg2_d_mss1_md_d_mirror}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed leaf \"{self.tg2_f_mss1_md_lizzard}\"",
			case_loggy.output
		)




		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed twig \"{self.tg3_d_mss1_md_d_mirror}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed twig \"{self.tg3_d_mss1_c_bbw}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed twig \"{self.d_msb3_a}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed twig \"{self.d_msb3_t}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed leaf \"{self.f_msb3_almost}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed leaf \"{self.tg3_f_mss1_md_lizzard}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:no_touch_flourish:Trimmed leaf \"{self.tg3_f_mss1_md_spock}\"",
			case_loggy.output
		)


		# First sprout
		self.assertTrue(os.path.isfile(self.f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))


		# First sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_goodbad))
		# First bough extra
		self.assertFalse(os.path.isfile(self.f_msb1_flower))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_lizzard))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_spock))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_d_mirror))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_bbw))


		# First sprout second bough
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_goodbad))
		# Second bough extra
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_lizzard))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_spock))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_md_d_mirror))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bbw))
		self.assertFalse(os.path.isdir(self.d_msb2_w))


		# First sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_goodbad))
		# Third bough extra
		self.assertFalse(os.path.isfile(self.f_msb3_almost))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_lizzard))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_spock))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_md_d_mirror))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_bbw))
		self.assertFalse(os.path.isdir(self.d_msb3_a))
		self.assertFalse(os.path.isdir(self.d_msb3_t))
















	def test_touch_flourish(self):

		sleep(1.1)
		self.test_case.loggy.init_name = "touch_flourish"


		self.fmake(self.tg1_f_mss1_md_spock, "vaporizes rock")
		os.makedirs(self.tg1_d_mss1_c_bbw)
		self.fmake(self.tg2_f_mss1_md_lizzard, "poisones spock")
		os.makedirs(self.tg2_d_mss1_md_d_mirror)
		self.fmake(self.tg3_f_mss1_md_lizzard, "poisones spock")
		self.fmake(self.tg3_f_mss1_md_spock, "vaporizes rock")
		os.makedirs(self.tg3_d_mss1_md_d_mirror)
		os.makedirs(self.tg3_d_mss1_c_bbw)



		# First sprout
		self.assertTrue(os.path.isfile(self.f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))


		# First sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_goodbad))
		# First bough extra
		self.assertFalse(os.path.isfile(self.f_msb1_flower))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_lizzard))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_spock))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_d_mirror))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_bbw))


		# First sprout second bough
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_goodbad))
		# Second bough extra
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_lizzard))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_spock))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md_d_mirror))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bbw))
		self.assertFalse(os.path.isdir(self.d_msb2_w))


		# First sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_goodbad))
		# Third bough extra
		self.assertFalse(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_lizzard))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_spock))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md_d_mirror))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_bbw))
		self.assertFalse(os.path.isdir(self.d_msb3_a))
		self.assertFalse(os.path.isdir(self.d_msb3_t))




		with self.assertLogs("touch_flourish", 10) as case_loggy:

			self.test_case = self.test_case()
			self.test_case.sync()




		self.no_loggy_levels(case_loggy.output, 30,40,50)


		self.assertIn(

			f"INFO:touch_flourish:Trimmed twig \"{self.tg1_d_mss1_c_bbw}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:touch_flourish:Trimmed leaf \"{self.tg1_f_mss1_md_spock}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:touch_flourish:Trimmed leaf \"{self.f_msb1_flower}\"",
			case_loggy.output
		)





		self.assertNotIn(

			f"INFO:touch_flourish:Trimmed twig \"{self.d_msb2_w}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:touch_flourish:Trimmed twig \"{self.tg2_d_mss1_md_d_mirror}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:touch_flourish:Trimmed leaf \"{self.tg2_f_mss1_md_lizzard}\"",
			case_loggy.output
		)




		self.assertIn(

			f"INFO:touch_flourish:Trimmed twig \"{self.tg3_d_mss1_md_d_mirror}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:touch_flourish:Trimmed twig \"{self.tg3_d_mss1_c_bbw}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:touch_flourish:Trimmed twig \"{self.d_msb3_a}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:touch_flourish:Trimmed twig \"{self.d_msb3_t}\"",
			case_loggy.output
		)
		self.assertNotIn(

			f"INFO:touch_flourish:Trimmed leaf \"{self.f_msb3_almost}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:touch_flourish:Trimmed leaf \"{self.tg3_f_mss1_md_lizzard}\"",
			case_loggy.output
		)
		self.assertIn(

			f"INFO:touch_flourish:Trimmed leaf \"{self.tg3_f_mss1_md_spock}\"",
			case_loggy.output
		)


		# First sprout
		self.assertTrue(os.path.isfile(self.f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))


		# First sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_goodbad))
		# First bough extra
		self.assertFalse(os.path.isfile(self.f_msb1_flower))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_lizzard))
		self.assertFalse(os.path.isfile(self.tg1_f_mss1_md_spock))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_d_mirror))
		self.assertFalse(os.path.isdir(self.tg1_d_mss1_c_bbw))


		# First sprout second bough
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_goodbad))
		# Second bough extra
		self.assertFalse(os.path.isfile(self.tg2_f_mss1_md_lizzard))
		self.assertTrue(os.path.isfile(self.tg2_f_mss1_md_spock))
		self.assertFalse(os.path.isdir(self.tg2_d_mss1_md_d_mirror))
		self.assertTrue(os.path.isdir(self.tg2_d_mss1_c_bbw))
		self.assertFalse(os.path.isdir(self.d_msb2_w))


		# First sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_scissors))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md_gc))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg3_f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_goodbad))
		# Third bough extra
		self.assertFalse(os.path.isfile(self.f_msb3_almost))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_lizzard))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_spock))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_md_d_mirror))
		self.assertFalse(os.path.isdir(self.tg3_d_mss1_c_bbw))
		self.assertFalse(os.path.isdir(self.d_msb3_a))
		self.assertFalse(os.path.isdir(self.d_msb3_t))








if	__name__ == "__main__" : unittest.main(verbosity=2)










import	re
from	os						import R_OK
from	os						import access	as osaccess
from	os						import path		as ospath
from	typing					import Optional
from	typing					import Callable
from	typing					import Generator
from	pygwarts.magical.chests	import KeyChest
from	pygwarts.magical.spells	import patronus
from	pygwarts.irma.access	import LibraryAccess








class LibraryVolume(KeyChest):

	"""
		irma.access core super class that represents both, the existent file to access and it's access
		dispatcher. For the first part of it's functionality, every LibraryVolume object must have field
		"location" set to a string that must point to existent file to be processed. Also optional field
		"inrange" must be a string that will be compiled to a regex that will match desired lines. It
		is optional because it is possible to manually set "inrange" by the argument for corresponding
		method "get_range" for a file processing. Originally designed idea implies "inrange" to be set
		as a datetime string, e.g. as LibraryContrib logging pattern, to be matched for desired lines,
		but it's not the only way and also the wildcard ".+" might be used. LibraryVolume object is the
		point for LibraryAccess object to access specified file, so this is the second part of it, that
		is maintained by main methods:
			is_located		- method that verifies LibraryVolume points to accessible file;
			get_range		- method that suggests which lines of file will be considered by matching
							provided compiled regex (might be passed as an argument);
			g_reading		- generator method that opens "location" file and yields every line;
			g_triggering	- generator method that identifies what handling to trigger for every line.
		In the volume accessing time, the LibraryVolume object will serve as a container for all data that
		will be handled for that volume. Optional callable "Annex" is reserved for a object that will
		organize full volume access and, probably, organize data inducing.
	"""


	location	:str
	inrange		:Optional[str]
	Annex		:Optional[Callable[..., str | None]]


	def __init__(self, *args, **kwargs)	:
		super().__init__(*args, **kwargs)


		if	isinstance(liblink := getattr(self, "_UPPER_LAYER", None), LibraryAccess):

			liblink(self, "volume")
			self.loggy.debug(f"Assigned to {liblink}")
		else:
			self.loggy.info(f"Volume {self} not assigned to any library")




	def is_located(self) -> str | None :

		"""
			Preparation method that verifies provided "location" filed points to an existent file to read
			from. Returns the "location" string if verification was successful, None otherwise.
		"""

		if	isinstance(getattr(self, "location", None), str):
			if	ospath.isfile(self.location):
				if	osaccess(self.location, R_OK):


					self.loggy.debug(f"Accessible \"{self.location}\" located")
					return self.location


				else:	self.loggy.debug(f"Location \"{self.location}\" is not accessible")
			else:		self.loggy.debug(f"Location \"{self.location}\" is invalid")
		else:			self.loggy.debug("Location was not provided")




	def get_range(self, inrange :str =None) -> re.Pattern | None :

		"""
			Preparation method that compiles special regex, which must define lines to be processed
			by match, from "inrange" string, which must be either specified as a field or as an
			argument (the last one takes priority). Returns compiled regex if "inrange" string has
			been found, or None otherwise. The wildcard ".+" might be specified for matching every
			line, but it is not done by default.
		"""

		match inrange:

			case str(): return re.compile(inrange)
			case None if isinstance(getattr(self, "inrange", None), str): return re.compile(self.inrange)

		self.loggy.info(f"Valid {self} range not provided")




	def g_reading(self) -> Generator[str,None,None] :

		"""
			Access method that relies on valid "location" field, which will be ensured by "is_located"
			method beforehand, opens it and generates every line read. Any exception during file
			accessing will be caught and logged. Doesn't supposed to return anything.
		"""

		if	self.is_located():
			try:

				with open(self.location) as volume:
					self.loggy.debug(f"Location \"{self.location}\" access granted")

					for line in volume : yield line.rstrip("\n")
			except	Exception as E:

				self.loggy.error(f"Location \"{self.location}\" access failed due to {patronus(E)}")




	def g_triggering(self, line :str, triggers: re.Pattern) -> Generator[str,None,None] :

		"""
			Access method that accepts two arguments - "line" string that was triggered and all "triggers"
			regex. When the "line" is triggered it must mean there are VolumeBookmark objects which
			"trigger" fields were found as current "line" substrings, which in turn means this "line"
			must be handled by that VolumeBookmark objects. For this to happen, "triggers" regex must
			comprise the triggers for VolumeBookmark objects. The main idea of current method is to
			find all possible "trigger" fields that will correspond to VolumeBookmark objects and yield it
			as strings. Doesn't supposed to return anything.
		"""

		if	isinstance(line, str):
			if	isinstance(triggers, re.Pattern):


				for triggered_group in triggers.finditer(line):
					for response in triggered_group.groups():
						if	response:


							self.loggy.debug(f"Triggered by \"{response}\"")
							yield response


			else:	self.loggy.debug(f"Invalid triggers \"{triggers}\"")
		else:		self.loggy.debug(f"Invalid line \"{line}\"")










from pathlib								import Path
from typing									import NoReturn
from pygwarts.magical.philosophers_stone	import Transmutable
from pygwarts.hagrid.thrivables				import Tree








class TwigProbe(Transmutable):

	"""
		hagrid.planting object, that represents simulation of catalog creation operation.
		The purpose of this object is to indicate the intention of real folders operations, before they
		will really affect the file system. There two things that this object __call__ actually does: logging
		probe info message if target folder doesn't exist (to be created), and logging debug message otherwise
		(already exist and no creation need).
		If a source folder doesn't exist at the start of operation, e.g. that might happen when processing
		some not small catalogs in multi-bough Flourish, the process will be stopped with corresponding log.
		This object's __call__ accepts arguments that implement the "planting interface":
			origin	- Tree object that is currently planting, used especially for emitting logging messages;
			sprout	- string that represents the current source root;
			branch	- Path object that represents current source folder;
			twig	- Path object that represents current operation source folder;
			bough	- Path object that represents current operation destination parent folder.
	"""

	def __call__(self, origin :Tree, sprout :str, branch :Path, twig :Path, bough :Path):


		if	not twig.is_symlink():
			target = bough.joinpath(twig.name)


			if	twig.is_dir():

				if	not target.is_dir() : origin.loggy.info(f"Twig \"{target}\" probe")
				else:	origin.loggy.debug(f"Passed probe \"{target}\"")
			else:		origin.loggy.debug(f"Branch \"{twig}\" not located")
		else:			origin.loggy.debug(f"Symbolic twig skipped")








class TwigThrive(Transmutable):

	"""
		hagrid.planting object, that represents catalog creating operation.
		The process consist of prechecking the existence of target folder and creating the target folder
		with all parents, in case of it doesn't exist (no recreation of already existent folders).
		The catalog creation function that used (Path.mkdir) doesn't preserve source file meta data.
		No return value for __call__ is assumed, also no handling for any Exception, that might be raised
		by catalog creation function, that must be handled by invoker object, e.g. Bloom object.
		If a source folder doesn't exist at the start of operation, e.g. that might happen when processing
		some not small catalogs in multi-bough Flourish, the process will be stopped with corresponding log.
		This object's __call__ accepts arguments that implement the "planting interface":
			origin	- Tree object that is currently planting, used especially for emitting logging messages;
			sprout	- string that represents the current source root;
			branch	- Path object that represents current source folder;
			twig	- Path object that represents current operation source folder;
			bough	- Path object that represents current operation destination parent folder.
	"""

	def __call__(self, origin :Tree, sprout :str, branch :Path, twig :Path, bough :Path) -> None | NoReturn :


		if	not twig.is_symlink():
			target = bough.joinpath(twig.name)


			if	twig.is_dir():
				if	not target.is_dir():


					target.mkdir(parents=True)
					origin.loggy.info(f"Thrived twig \"{target}\"")
				else:
					origin.loggy.debug(f"Passed twig \"{target}\"")
			else:	origin.loggy.debug(f"Branch \"{twig}\" not located")
		else:		origin.loggy.debug(f"Symbolic twig skipped")







